/**
 * Code released under WTFPL [http://sam.zoy.org/wtfpl/]
 * 
 * @author Arnaud Pichery [http://coderpeon.ovh.org]
 */
package com.flexspy
{
	import flash.display.DisplayObject;
	import flash.events.Event;
	
	import mx.collections.ArrayCollection;
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.controls.Alert;
	import mx.controls.Button;
	import mx.controls.Tree;
	import mx.core.Container;
	import mx.core.UIComponent;
	import mx.events.CloseEvent;
	import mx.events.CollectionEvent;
	import mx.events.FlexEvent;
	import mx.managers.PopUpManager;
	import mx.containers.TitleWindow;
	import mx.core.Application;
	import flash.events.KeyboardEvent;
	import mx.managers.FocusManager;
	import flash.events.MouseEvent;
	import mx.events.ListEvent;
	import mx.containers.HDividedBox;
	import mx.containers.TabNavigator;

	/**
	 * <p>Displays details about a component and its children in a window.</p>
	 * 
	 * <p>To display the window, you call the method <code>show(root: DisplayObject, modal: Boolean)</code>.</p>
	 */
	public class ComponentTreeWnd extends TitleWindow
	{
		// UI component displaying the tree of all components.
		private var _componentTree: Tree;
		
		// UI component displaying the properties of a component.
		private var _componentProperties: ComponentProperties;

		// UI component displaying the styles of a component.
		private var _componentStyles: ComponentStyles;
		
		// Root component of the tree
		private var _root: DisplayObject;
		
		// Indicates whether the window is displayed in a modal mode.
		private var _modal: Boolean;
		
		// Only instance of our window. Non-null when window is displayed, null otherwise.
		private static var _instance: ComponentTreeWnd;
		
	    /**
	     * This method is not intented to be used. Use <code>show</code> method instead.
	     */
		public function ComponentTreeWnd() {
			super();
			
			// Singleton pattern
			if (_instance != null)
				throw new Error("Only one instance of ComponentTreeWnd can be created.");
			_instance = this;
			
			// properties
			this.layout = "vertical";
			this.title = "Component Tree";
			this.width = 900;
			this.height = 678;
			this.showCloseButton = true;

			// events
			this.addEventListener("close", closeWindow);
			this.addEventListener("creationComplete", onCreationComplete);
			this.addEventListener(KeyboardEvent.KEY_UP, onKeyPressed);
		}
		
		override protected function createChildren(): void {
			super.createChildren();
			
			var vbox: mx.containers.HDividedBox = new mx.containers.HDividedBox();
			vbox.percentWidth = 100.0;
			vbox.percentHeight = 100.0;
			
			_componentTree = new Tree();
			_componentTree.percentWidth = 50.0;
			_componentTree.percentHeight = 100.0;
			_componentTree.addEventListener(ListEvent.CHANGE, onItemSelected);
			_componentTree.iconFunction = getTreeNodeIcon;
			vbox.addChild(_componentTree);
			
			var _rightTab: TabNavigator = new TabNavigator();
			_rightTab.historyManagementEnabled = false;
			_rightTab.percentWidth = 50.0;
			_rightTab.percentHeight = 100.0;
			vbox.addChild(_rightTab);
			
			_componentProperties = new ComponentProperties();
			_componentProperties.percentWidth = 100.0;
			_componentProperties.percentHeight = 100.0;
			_rightTab.addChild(_componentProperties);

			_componentStyles = new ComponentStyles();
			_componentStyles.percentWidth = 100.0;
			_componentStyles.percentHeight = 100.0;
			_rightTab.addChild(_componentStyles);

			this.addChild(vbox);

			
			var cbar: mx.containers.Box = new mx.containers.HBox();
   			cbar.setStyle("horizontalAlign", "right");
   			cbar.percentWidth = 100.0;
			if (!_modal) {
				// Add refresh button when window is not modal
				var refreshButton: Button = new Button();
				refreshButton.label = "Refresh";
				refreshButton.addEventListener(MouseEvent.CLICK, onRefreshButtonClicked);
				cbar.addChild(refreshButton);
			}
			var closeButton: Button = new Button();
			closeButton.label = "Close";
			closeButton.addEventListener("click", closeWindow);
			cbar.addChild(closeButton);
			this.addChild(cbar);
		}
		
		/**
		 * Displays the tree of the specified DisplayObject (its children, the children of its children, etc.)
		 * 
		 * @param root Root of the displayed tree. If it is null, the current application is used.
		 * @param modal true to display a modal window (default), false to display a modeless window.
		 */
		public static function show(root: DisplayObject = null, modal: Boolean = true): void {
			if (root == null) {
				root = DisplayObject(Application.application);
			}
						
			if (_instance == null) {
				_instance = ComponentTreeWnd(PopUpManager.createPopUp(root, ComponentTreeWnd, modal));
				_instance._root = root;
				_instance._modal = modal;
				PopUpManager.centerPopUp(_instance);
			} else if (!_instance._modal) {
				// Modeless window displayed, update the root component and refresh.
				_instance._root = root;
				_instance.initWindow();
			} else {
				// Modal window, do nothing.
			}
		}

		private function closeWindow(event: Event = null): void {
			PopUpManager.removePopUp(this);
			_instance = null;			
		}
		
		private function onKeyPressed(event: KeyboardEvent): void {
			if (event.keyCode == 27) { // 27 = ESC key
				closeWindow();
			}
		}

		private function onCreationComplete(event: Event): void {
			initWindow();
			focusManager.setFocus(_componentTree);
		}

		private function onRefreshButtonClicked(event: Event): void {
			initWindow();
		}
		
		/**
		 * Fill-in the component tree with the root component.
		 * 
		 * This method is called the first time the window is created, 
		 * and when users click the "refresh" button.
		 */
		private function initWindow(): void {
			// Compute the component tree and display it
			var rootItem: IComponentTreeItem = new ComponentTreeItem(_root);
			var treeNodes: ArrayCollection = new ArrayCollection();
			treeNodes.addItem(rootItem);
			_componentTree.dataProvider = treeNodes;
			_componentTree.validateNow();
			_componentTree.expandItem(rootItem, true);
			_componentProperties.showComponentProperties(null);
			_componentStyles.showComponentStyles(null);
		}
		
        private function onItemSelected(event: Event): void {
			var item: ComponentTreeItem = event.currentTarget.selectedItem as ComponentTreeItem;
			var displayObject: DisplayObject = (item == null) ? null : item.displayObject;
			_componentProperties.showComponentProperties(displayObject);
			_componentStyles.showComponentStyles(displayObject);
        }
        
		private function getTreeNodeIcon(item:Object):Class{
		   return IComponentTreeItem(item).icon;
		}

		private static function onRegistrationKeyPressed(event: KeyboardEvent): void {
			if (event.ctrlKey && event.shiftKey && event.keyCode == 84) {
					ComponentTreeWnd.show(Application(event.currentTarget), true);
			}
		}
	}
}
